# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING, Any, Optional

import dependency_injector.providers as providers
from PySide2.QtCore import QPointF
from PySide2.QtGui import QPainterPath, QPainterPathStroker
from PySide2.QtWidgets import QGraphicsItem, QGraphicsPathItem

from .graphics_connection_painter import GraphicsConnectionPainterFactory

if TYPE_CHECKING:
    from PySide2.QtGui import QPainter
    from PySide2.QtCore import QRectF
    from PySide2.QtWidgets import QWidget, QStyleOptionGraphicsItem
    from .graphics_port import GraphicsPort
    from .graphics_connection_painter import GraphicsConnectionPainter


class GraphicsConnection(QGraphicsPathItem):
    """Class representing a line between two points (or ports).

    Can be used to drag a connection between two node's ports, for example.

    Attributes:
        color: Color used to draw this connection.
        start: Start point of this connection.
        end: End point of this connection.
        start_graphics_port: GraphicsPort object attached to the start of this
            connection (if any).
        end_graphics_port: GraphicsPort object attached to the end of this
            connection (if any).

    """

    def __init__(
        self, painter_factory: "providers.Factory", parent: "QGraphicsItem" = None,
    ):
        super().__init__(parent)

        # A Connection can be selectable
        self.setFlag(QGraphicsItem.ItemIsSelectable)

        self.width = 4
        self.clickable_margin = 20

        self.__start = QPointF(0, 0)
        self.__end = QPointF(0, 0)

        self.__start_graphics_port: Optional["GraphicsPort"] = None
        self.__end_graphics_port: Optional["GraphicsPort"] = None

        self._painter_factory = painter_factory
        self._graphics_connection_painter = painter_factory(graphics_connection=self)

        # Draw connections always on bottom
        self.setZValue(-1)

        self._update_path()

    @property
    def painter(self) -> "GraphicsConnectionPainter":
        return self._graphics_connection_painter

    @property
    def start(self) -> "QPointF":
        """Returns the start position coordinate of this connection."""
        return (
            self.__start
            if not self.__start_graphics_port
            else self.__start_graphics_port.pos()
        )

    @start.setter
    def start(self, position: "QPointF"):
        """Sets a new start position coordinate for this connection.

        When a position is set this way, the connection is disconnected from any
        GraphicsPort it may be connected to (Because we're moving the connection away
        from the port)."""
        self.__start = position

        if self.__start_graphics_port:
            self.__start_graphics_port.remove_connection(self)

        self.__start_graphics_port = None

        self._update_path()

    @property
    def end(self) -> "QPointF":
        """Returns the end position of the connection."""
        return (
            self.__end
            if not self.__end_graphics_port
            else self.__end_graphics_port.pos()
        )

    @end.setter
    def end(self, position: "QPointF"):
        """Sets a new end position coordinate for this connection.

        When a position is set this way, the connection is disconnected from any
        GraphicsPort it may be connected to (Because we're moving the connection away
        from the port)."""
        self.__end = position

        if self.__end_graphics_port:
            self.__end_graphics_port.remove_connection(self)

        self.__end_graphics_port = None

        self._update_path()

    @property
    def start_graphics_port(self) -> Optional["GraphicsPort"]:
        """Returns the port connected to the start of this connection."""
        return self.__start_graphics_port

    @start_graphics_port.setter
    def start_graphics_port(self, graphics_port: Optional["GraphicsPort"]):
        """Sets the start of this connection to the `port` position.

        The connection adopts the color of the start port.
        """
        if not graphics_port:
            if self.__start_graphics_port:
                self.__start_graphics_port.remove_connection(self)

            self.start = self.__start
            self.__start_graphics_port = None
        else:
            # Updates the start position
            self.__start = graphics_port.pos()

            # Assigns a new start port
            self.__start_graphics_port = graphics_port
            self.__start_graphics_port.add_connection(self)

            # The connection adopts the color of the port
            self.painter.color = graphics_port.painter.color

        self._update_path()

    @property
    def end_graphics_port(self) -> Optional["GraphicsPort"]:
        """Returns the port connected to the end of this connection."""
        return self.__end_graphics_port

    @end_graphics_port.setter
    # @log_on_end(INFO, "{self} connected to End Port {port}")
    def end_graphics_port(self, graphics_port: "GraphicsPort"):
        """Sets the end of this connection to the `port` position."""
        if not graphics_port:
            if self.__end_graphics_port:
                self.__end_graphics_port.remove_connection(self)

            self.end = self.__end
            self.__end_graphics_port = None
        else:
            # Updates the end position
            self.__end = graphics_port.pos()

            # Assigns a new end port
            self.__end_graphics_port = graphics_port
            self.__end_graphics_port.add_connection(self)

        self._update_path()

    def is_connected(self) -> bool:
        """Returns if the start and end ports are connected."""
        return bool(self.__start_graphics_port and self.__end_graphics_port)

    def _update_path(self):
        """Creates a new bezier path from `self.start` to `self.end`."""
        path = QPainterPath(self.start)

        diffx = self.end.x() - self.start.x()

        c0x = self.start.x() + (diffx / 3)
        c0y = self.start.y()
        c1x = self.end.x() - (diffx / 3)
        c1y = self.end.y()

        path.cubicTo(c0x, c0y, c1x, c1y, self.end.x(), self.end.y())

        self.setPath(path)

    def itemChange(self, change: "QGraphicsItem.GraphicsItemChange", value: Any) -> Any:
        """Checks if any property of the connection has changed.

        In this case, we change the color of the connection when it's selected.
        """
        if change == self.ItemSelectedChange:
            self.painter.highlight_color(value)
            return value

        return super().itemChange(change, value)

    def boundingRect(self) -> "QRectF":
        return self.shape().boundingRect().normalized()

    def shape(self) -> "QPainterPath":
        path_stroker = QPainterPathStroker()
        path_stroker.setWidth(self.width + self.clickable_margin)
        return path_stroker.createStroke(self.path())

    def paint(
        self,
        painter: "QPainter",
        option: "QStyleOptionGraphicsItem",
        widget: "QWidget" = None,
    ):
        """Paints the connection between the start and end points."""

        self._update_path()

        self._graphics_connection_painter.paint(painter, option, widget)

    def __getstate__(self):
        return {
            "start": self.__start,
            "end": self.__end,
            "start_graphics_port": self.__start_graphics_port,
            "end_graphics_port": self.__end_graphics_port,
        }

    def __setstate__(self, new_state: dict):
        self.__start_graphics_port = new_state["start_graphics_port"]
        self.__end_graphics_port = new_state["end_graphics_port"]
        self.__start = new_state["start"]
        self.__end = new_state["end"]

    def __reduce__(self):
        return (GraphicsConnection, (self._painter_factory,), self.__getstate__())

    def __eq__(self, other) -> bool:
        return (
            isinstance(other, self.__class__)
            and self.start == other.start
            and self.end == other.end
            and self.start_graphics_port == other.start_graphics_port
            and self.end_graphics_port == other.end_graphics_port
        )

    def __str__(self) -> str:
        return (
            f"{type(self).__name__} {self.start} ({self.start_graphics_port})"
            f" {self.end} ({self.end_graphics_port})"
        )


GraphicsConnectionFactory = providers.Factory(
    GraphicsConnection, painter_factory=GraphicsConnectionPainterFactory.delegate(),
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from PySide2.QtCore import QSize
from PySide2.QtWidgets import QDialog, QVBoxLayout, QGroupBox

from dial_gui.widgets.log import LoggerTextboxFactory
from dial_core.utils import log
import logging

from .plugins_table import PluginsTableWidgetFactory

if TYPE_CHECKING:
    from .plugins_table import PluginsTableWidget


class PluginManagerDialog(QDialog):
    def __init__(self, plugins_table_widget: "PluginsTableWidget", parent=None):
        super().__init__(parent)

        self.setWindowTitle("Plugin Manager")

        self.__plugins_table_widget = plugins_table_widget
        self.__plugins_table_widget.setParent(self)

        self.__actions_output_textarea = LoggerTextboxFactory(
            formatter=logging.Formatter(
                "%(asctime)s - %(levelname)s: %(message)s", "%H:%M:%S"
            ),
            parent=self,
        )
        log.get_logger("plugins").addHandler(self.__actions_output_textarea)

        self.__main_layout = QVBoxLayout()

        actions_output_group = QGroupBox("Commands output")
        actions_output_layout = QVBoxLayout()
        actions_output_layout.addWidget(self.__actions_output_textarea)
        actions_output_group.setLayout(actions_output_layout)

        self.__main_layout.addWidget(self.__plugins_table_widget)
        self.__main_layout.addWidget(actions_output_group)

        self.setLayout(self.__main_layout)

    def sizeHint(self) -> "QSize":
        """Preferred size of this dialog."""
        return QSize(600, 400)


PluginManagerDialogFactory = providers.Factory(
    PluginManagerDialog, plugins_table_widget=PluginsTableWidgetFactory
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

"""
This package includes several widgets that can be used to display a logging dialog.
"""

from .logger_dialog import LoggerDialogFactory
from .logger_textbox import LoggerTextboxFactory

__all__ = ["LoggerDialogFactory", "LoggerTextboxFactory"]



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING
from PySide2.QtWidgets import QMenu, QAction

import dependency_injector.providers as providers

from dial_gui.widgets.log import LoggerDialogFactory

if TYPE_CHECKING:
    from dial_gui.widgets.log import LoggerDialog
    from PySide2.QtWidgets import QWidget


class WindowsMenu(QMenu):
    def __init__(self, logger_dialog: "LoggerDialog", parent: "QWidget" = None):
        super().__init__("&Windows", parent)

        self.__logger_dialog = logger_dialog

        self._show_log_act = QAction("Show log", self)
        self._show_log_act.triggered.connect(self.__toggle_logger_dialog)

        self.addAction(self._show_log_act)

    def __toggle_logger_dialog(self):
        self.__logger_dialog.show()


WindowsMenuFactory = providers.Factory(WindowsMenu, logger_dialog=LoggerDialogFactory)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

"""This package includes the necessary classes for creating the main window for the
application."""

from .main_window import MainWindowFactory

__all__ = ["MainWindowFactory"]



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from .plugin_manager_dialog import PluginManagerDialog, PluginManagerDialogFactory

__all__ = ["PluginManagerDialog", "PluginManagerDialogFactory"]



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QAction, QMenu

from dial_core.node_editor import Node, NodeRegistrySingleton

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget
    from dial_core.node_editor import NodeRegistry


class NodesMenu(QMenu):
    node_created = Signal(Node)

    def __init__(
        self, node_registry: "NodeRegistry", parent: "QWidget" = None,
    ):
        super().__init__("&Nodes", parent)

        self.setTearOffEnabled(True)

        for node_name, factory in node_registry.nodes.items():
            action = QAction(node_name, self)

            action.triggered.connect(
                lambda _=False, factory=factory: self.node_created.emit(factory())
            )

            self.addAction(action)


NodesMenuFactory = providers.Factory(NodesMenu, node_registry=NodeRegistrySingleton)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

from PySide2.QtWidgets import QVBoxLayout, QWidget

from .node_editor_view import NodeEditorView

if TYPE_CHECKING:
    from PySide2.QtWidgets import QTabWidget
    from dial_core.project import ProjectManager
    from dial_gui.project import ProjectGUI


class NodeEditorWindow(QWidget):
    def __init__(
        self,
        tabs_widget: "QTabWidget",
        project_manager: "ProjectManager",
        parent: "QWidget" = None,
    ):
        super().__init__(parent)

        self.__project_manager = project_manager

        self.__main_layout = QVBoxLayout()

        self.__node_editor_view = NodeEditorView(tabs_widget, parent=self)
        self.__graphics_scene = self.__project_manager.active.graphics_scene

        self.__node_editor_view.setScene(self.__graphics_scene)

        self.__main_layout.addWidget(self.__node_editor_view)
        self.__main_layout.setContentsMargins(0, 0, 0, 0)

        self.setLayout(self.__main_layout)

        self.__project_manager.active_project_changed.connect(
            self.__active_project_changed
        )

        self.show()
        self.__active_project_changed(self.__project_manager.active)

    def __active_project_changed(self, project: "ProjectGUI"):
        self.__node_editor_view.disconnect(self.__graphics_scene)
        self.__node_editor_view.setScene(project.graphics_scene)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import Optional

import os
import json

from PySide2.QtCore import QStandardPaths
import pkg_resources


def version() -> str:
    """Returns the current version of the application."""
    return pkg_resources.require("dial-gui")[0].version


def config_directory() -> str:
    config_directory = (
        QStandardPaths.writableLocation(QStandardPaths.ConfigLocation)
        + os.path.sep
        + "dial"
    )

    if not os.path.isdir(config_directory):
        os.mkdir(config_directory)

    return config_directory


def plugins_directory() -> str:
    plugins_directory = config_directory() + os.path.sep + "plugins"

    if not os.path.isdir(plugins_directory):
        os.mkdir(plugins_directory)

    return plugins_directory


def plugins_install_directory() -> str:
    plugins_install_directory = plugins_directory() + os.path.sep + "site-packages"

    if not os.path.isdir(plugins_install_directory):
        os.mkdir(plugins_install_directory)

    return plugins_install_directory


def installed_plugins_file() -> str:
    return plugins_directory() + os.path.sep + "plugins.json"


def installed_plugins_file_content() -> Optional[dict]:
    with open(installed_plugins_file()) as plugins_file:
        return json.load(plugins_file)

    return None



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
"""
Functions to check that python versions, libraries... used by the program are correct.
"""


import json
import os
import signal
import sys
from typing import TYPE_CHECKING

import dial_core
import pkg_resources
from dial_core.plugin import PluginManagerSingleton
from dial_core.utils import log

from . import application

if TYPE_CHECKING:
    import argparse


LOGGER = log.get_logger(__name__)


def initialize(args: "argparse.Namespace"):
    """Performs all the necessary steps before running the application. This checks
    python version, installed modules, graphics configurations, initialize logging
    system...

    Raises:
        ImportError: If couldn't import a necessary module.
        SystemError: If the Python version isn't compatible.
    """
    try:

        dial_core.utils.initialization.initialize(args)
        __gui_initialization(args)

    except (ImportError, SystemError) as err:
        LOGGER.exception(err)

        from dial_gui.utils import tkinter as dial_tkinter

        dial_tkinter.showerror(str(err))
        sys.exit(1)


def __gui_initialization(args: "argparse.Namespace"):
    """Performs all the initialization of the GUI components.

    Args:
        args: App configuration namespace."""
    # State the signals handled by this application
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # Initialize PySide2
    from PySide2.QtWidgets import QApplication

    app = QApplication()
    app.setApplicationName("dial")
    app.aboutToQuit.connect(exit_application)

    __plugins_initialization(args)


def __plugins_initialization(args: "argparse.Namespace"):
    plugins_install_abs_path = os.path.abspath(application.plugins_install_directory())

    sys.path.append(plugins_install_abs_path)
    pkg_resources.working_set.add_entry(plugins_install_abs_path)

    LOGGER.info("%s added to sys.path", plugins_install_abs_path)

    plugins_manager = PluginManagerSingleton()

    LOGGER.debug("Installed plugins: %s", plugins_manager.installed_plugins)

    for plugin in plugins_manager.installed_plugins.values():
        try:
            if plugin.active:
                plugin.load()
        except ModuleNotFoundError as err:
            LOGGER.exception(err)


def exit_application():
    # Save plugin manager current state
    plugins_manager = PluginManagerSingleton()

    with open(application.installed_plugins_file(), "w") as plugins_file:
        json.dump(plugins_manager.to_dict(), plugins_file, indent=2)

    # Close the PySide2 application
    from PySide2.QtWidgets import QApplication

    QApplication.quit()

    LOGGER.info("Dial closed.")



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING, List

import dependency_injector.providers as providers
from dial_core.node_editor import Node, Scene, SceneFactory
from dial_core.utils import log
from PySide2.QtWidgets import QGraphicsItem, QGraphicsScene

from .graphics_connection import GraphicsConnection
from .graphics_node import GraphicsNode, GraphicsNodeFactory
from .graphics_scene_painter import GraphicsScenePainterFactory

if TYPE_CHECKING:
    from PySide2.QtWidgets import QObject
    from PySide2.QtCore import QRectF
    from PySide2.QtGui import QPainter

LOGGER = log.get_logger(__name__)


class GraphicsScene(QGraphicsScene):
    def __init__(
        self,
        scene: "Scene",
        painter_factory: "providers.Factory",
        parent: "QObject" = None,
    ):
        super().__init__(parent)

        self.__scene = scene
        self.__graphics_nodes: List["GraphicsNode"] = []

        # Painter
        self._painter_factory = painter_factory
        self._graphics_scene_painter = painter_factory(graphics_scene=self)

        # Populate the graphics scene
        for node in self.__scene:
            graphics_node = self.__create_graphics_node_from(node)
            self.__graphics_nodes.append(graphics_node)
            super().addItem(graphics_node)

    @property
    def scene(self):
        """Returns the scene attached to this graphics scene."""
        return self.__scene

    def addItem(self, item: "QGraphicsItem"):
        if isinstance(item, GraphicsNode):
            self.__add_graphics_node(item)
            return

        super().addItem(item)

    def removeItem(self, item: "QGraphicsItem"):
        if isinstance(item, GraphicsNode):
            self.__remove_graphics_node(item)
            return

        if isinstance(item, GraphicsConnection):
            self.__remove_graphics_connection(item)
            return

        super().removeItem(item)

    def drawBackground(self, painter: "QPainter", rect: "QRectF"):
        """Draws the background for the scene."""
        super().drawBackground(painter, rect)

        self._graphics_scene_painter.drawBackground(painter, rect)

    def __add_graphics_node(self, graphics_node: "GraphicsNode"):
        self.__scene.add_node(graphics_node._node)
        self.__graphics_nodes.append(graphics_node)

        super().addItem(graphics_node)

    def __remove_graphics_node(self, graphics_node: "GraphicsNode"):
        try:
            self.__graphics_nodes.remove(graphics_node)
            self.__scene.remove_node(graphics_node._node)

            for port in list(graphics_node.inputs.values()) + list(
                graphics_node.outputs.values()
            ):
                for connection in port.graphics_connections:
                    self.__remove_graphics_connection(connection)

            super().removeItem(graphics_node)

        except ValueError:
            pass

    def __remove_graphics_connection(self, graphics_connection: "GraphicsConnection"):
        graphics_connection.start_graphics_port = None
        graphics_connection.end_graphics_port = None
        super().removeItem(graphics_connection)

    def __create_graphics_node_from(self, node: "Node"):
        return GraphicsNodeFactory(node, graphics_scene=self)

    def __getstate__(self):
        LOGGER.debug("Saving scene:\n%s", self.__scene)
        return {"scene": self.__scene, "graphics_nodes": self.__graphics_nodes}

    def __setstate__(self, new_state: dict):
        """Composes a GraphicsScene object from a pickled dict."""
        self.clear()

        self.__scene = new_state["scene"]
        self.__graphics_nodes = new_state["graphics_nodes"]

        LOGGER.debug("Loading scene:\n%s", self.__scene)

        for graphics_node in self.__graphics_nodes:
            super().addItem(graphics_node)

            for graphics_port in list(graphics_node.inputs.values()) + list(
                graphics_node.outputs.values()
            ):
                for graphics_connection in graphics_port.graphics_connections:
                    # TODO: Solve items duplication with this approach
                    self.addItem(graphics_connection)

        self.update()

    def __reduce__(self):
        # Return an empty scene (Because the real scene will be restored later)
        return (
            GraphicsScene,
            (Scene(), self._painter_factory,),
            self.__getstate__(),
        )


GraphicsSceneFactory = providers.Factory(
    GraphicsScene,
    scene=SceneFactory,
    painter_factory=GraphicsScenePainterFactory.delegate(),
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from enum import Enum
from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from PySide2.QtCore import QRectF, Qt
from PySide2.QtGui import QBrush, QPainter, QPen
from PySide2.QtWidgets import QGraphicsItem, QGraphicsTextItem

from .type_colors import TypeColor

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget, QStyleOptionGraphicsItem
    from .graphics_port import GraphicsPort


class GraphicsPortPainter:
    """The GraphicsPortPainter class provides a set of functions and configurations for
        painting a GraphicsPort object.
    """

    class DrawingState(Enum):
        Normal = 0
        Target = 1

    class PortNamePosition(Enum):
        Left = 0
        Right = 1

    drawing_state = DrawingState.Normal
    target_port_type = None

    def __init__(
        self, graphics_port: "GraphicsPort", port_name_position: "PortNamePosition"
    ):
        self.__graphics_port = graphics_port
        self.__port_name = QGraphicsTextItem(parent=graphics_port)
        self.__port_name.setPlainText(graphics_port._port.name)
        self.__port_name.setDefaultTextColor("#FFFFFF")
        self.__port_name.setFlag(QGraphicsItem.ItemStacksBehindParent)

        self.port_name_position = port_name_position

        # Colors/Pens/Brushes

        self.__color = TypeColor.get_color_for(graphics_port._port.port_type)

        self.__outline_pen = QPen(self.__color.darker())
        self.__outline_pen.setWidthF(2)
        self.__background_brush = QBrush(self.__color)

        self.__dashed_outline_pen = QPen(self.__color)
        self.__dashed_outline_pen.setStyle(Qt.DashLine)
        self.__dashed_outline_pen.setWidth(2)

    @property
    def port_name_position(self) -> "PortNamePosition":
        return self.__port_name_position

    @port_name_position.setter
    def port_name_position(self, position: "PortNamePosition"):
        if position == self.PortNamePosition.Left:
            self.__port_name.setPos(-self.__port_name.boundingRect().width() - 3, 1)
        elif position == self.PortNamePosition.Right:
            self.__port_name.setPos(3, 1)

        self.__port_name_position = position

    @property
    def color(self):
        return self.__color

    def paint_area(self):
        return QRectF(
            -self.__graphics_port.radius,
            -self.__graphics_port.radius,
            2 * self.__graphics_port.radius,
            2 * self.__graphics_port.radius,
        )

    def paint(
        self,
        painter: "QPainter",
        option: "QStyleOptionGraphicsItem",
        widget: "QWidget" = None,
    ):
        if (
            GraphicsPortPainter.drawing_state == GraphicsPortPainter.DrawingState.Target
            and self.__graphics_port.port_type == GraphicsPortPainter.target_port_type
        ):
            painter.setPen(self.__dashed_outline_pen)
            painter.setBrush(Qt.NoBrush)
            painter.drawEllipse(self.__graphics_port.boundingRect())

        painter.setPen(self.__outline_pen)
        painter.setBrush(self.__background_brush)
        painter.drawEllipse(self.paint_area())


GraphicsPortPainterFactory = providers.Factory(
    GraphicsPortPainter, port_name_position=GraphicsPortPainter.PortNamePosition.Left
)

InputGraphicsPortPainterFactory = providers.Factory(
    GraphicsPortPainter, port_name_position=GraphicsPortPainter.PortNamePosition.Left
)

OutputGraphicsPortPainterFactory = providers.Factory(
    GraphicsPortPainter, port_name_position=GraphicsPortPainter.PortNamePosition.Right
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from dial_gui.project import ProjectManagerGUISingleton
from PySide2.QtCore import Signal
from PySide2.QtGui import QKeySequence
from PySide2.QtWidgets import QAction, QMenu

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget
    from dial_core.project import ProjectManager


class FileMenu(QMenu):
    """The class FileMenu provides a configured menu for the basic File operations
    (Open/Close/Save projects, exit program...)."""

    quit = Signal()

    def __init__(self, project_manager: "ProjectManager", parent: "QWidget" = None):
        super().__init__("&File", parent)

        self.__project_manager = project_manager

        # Add actions
        self._new_project_act = QAction("New project", self)
        self._new_project_act.setShortcut(QKeySequence.New)
        self._new_project_act.triggered.connect(self.__project_manager.new_project)

        self._open_project_act = QAction("Open project", self)
        self._open_project_act.setShortcut(QKeySequence.Open)
        self._open_project_act.triggered.connect(self.__project_manager.open_project)

        self._save_project_act = QAction("Save project", self)
        self._save_project_act.setShortcut(QKeySequence.Save)
        self._save_project_act.triggered.connect(
            lambda: self.__project_manager.save_project(self.__project_manager.active)
        )

        self._save_project_as_act = QAction("Save project as...", self)
        self._save_project_as_act.triggered.connect(
            lambda: self.__project_manager.save_project_as(
                self.__project_manager.active
            )
        )

        self._close_project_act = QAction("Close project", self)
        self._close_project_act.triggered.connect(
            lambda: self.__project_manager.close_project(self.__project_manager.active)
        )

        self._quit_act = QAction("Quit", self)
        self._quit_act.setShortcut(QKeySequence.Quit)
        self._quit_act.triggered.connect(lambda: self.quit.emit())

        # Configure menu
        self.addAction(self._new_project_act)
        self.addAction(self._open_project_act)
        self.addAction(self._save_project_act)
        self.addAction(self._save_project_as_act)
        self.addAction(self._close_project_act)
        self.addSeparator()
        self.addAction(self._quit_act)


FileMenuFactory = providers.Factory(
    FileMenu, project_manager=ProjectManagerGUISingleton
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from PySide2.QtCore import QUrl
from PySide2.QtGui import QDesktopServices
from PySide2.QtWidgets import QAction, QMenu

from dial_gui.utils import application
from dial_gui.widgets.plugin import PluginManagerDialogFactory

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget
    from dial_gui.widgets.plugin import PluginManagerDialog


class PluginsMenu(QMenu):
    def __init__(
        self, plugin_manager_dialog: "PluginManagerDialog", parent: "QWidget" = None
    ):
        super().__init__("&Plugins", parent)

        self.__plugin_manager_dialog = plugin_manager_dialog

        self._open_plugin_manager_act = QAction("Open plugin manager...", self)
        self._open_plugin_manager_act.triggered.connect(self.__show_plugin_manager)

        self._open_plugins_directory_act = QAction(
            'Open "plugins" directory in explorer...', self
        )
        self._open_plugins_directory_act.triggered.connect(
            self.__open_plugins_directory
        )

        self.addAction(self._open_plugin_manager_act)
        self.addAction(self._open_plugins_directory_act)

    def __show_plugin_manager(self):
        self.__plugin_manager_dialog.show()

    def __open_plugins_directory(self):
        plugins_directory = application.plugins_directory()

        QDesktopServices.openUrl(QUrl(plugins_directory, QUrl.TolerantMode))


PluginsMenuFactory = providers.Factory(
    PluginsMenu, plugin_manager_dialog=PluginManagerDialogFactory
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

import math
from typing import TYPE_CHECKING, List, Tuple

import dependency_injector.providers as providers
from PySide2.QtCore import QLine, QRect, QRectF
from PySide2.QtGui import QColor, QPainter, QPen

if TYPE_CHECKING:
    from .graphics_scene import GraphicsScene


class GraphicsScenePainter:
    def __init__(self, graphics_scene: "GraphicsScene"):
        self.__graphics_scene = graphics_scene

        self.width = 64000
        self.height = 64000

        self.__graphics_scene.setSceneRect(
            -self.width // 2, -self.height // 2, self.width, self.height,
        )

        self.grid_size = 20
        self.grid_squares = 5

        # Colors/Pens/Brushes
        self._color_background = QColor("#393939")
        self._color_light = QColor("#2f2f2f")
        self._color_dark = QColor("#292929")

        self._pen_light = QPen(self._color_light)
        self._pen_light.setWidth(1)

        self._pen_dark = QPen(self._color_dark)
        self._pen_dark.setWidth(2)

        self.__graphics_scene.setBackgroundBrush(self._color_background)

    def drawBackground(self, painter: "QPainter", rect: "QRectF"):
        grid_rect = self.__calculate_grid_boundaries(rect)

        # Compute all lines to be drawn
        light_lines, dark_lines = self.__calculate_grid_lines(grid_rect)

        # Draw all lines
        painter.setPen(self._pen_light)
        painter.drawLines(light_lines)

        painter.setPen(self._pen_dark)
        painter.drawLines(dark_lines)

    def __calculate_grid_boundaries(self, rect: "QRectF") -> "QRect":
        """Calculates the grid boundaries from the rect."""
        # Get grid boundaries
        left = int(math.floor(rect.left()))
        right = int(math.ceil(rect.right()))
        top = int(math.floor(rect.top()))
        bottom = int(math.ceil(rect.bottom()))

        # Anchor left and top to the closest grid line
        left = left - (left % self.grid_size)
        top = top - (top % self.grid_size)

        return QRect(left, top, right - left + 2, bottom - top + 2)

    def __calculate_grid_lines(self, grid_rect: "QRect") -> Tuple[List, List]:
        """Calculates the coordinates of the horizontal/vertical lines to be drawn.

        Args:
            grid_rect: Rectangle representing the grid boundaries.

        Returns:
            Lists of light lines and dark lines to draw.
        """
        light_lines = []
        dark_lines = []

        # Calculate horizontal lines
        for x in range(grid_rect.left(), grid_rect.right(), self.grid_size):
            if x % (self.grid_size * self.grid_squares) != 0:
                light_lines.append(QLine(x, grid_rect.top(), x, grid_rect.bottom()))
            else:
                dark_lines.append(QLine(x, grid_rect.top(), x, grid_rect.bottom()))

        # Calculate vertical lines
        for y in range(grid_rect.top(), grid_rect.bottom(), self.grid_size):
            if y % (self.grid_size * self.grid_squares) != 0:
                light_lines.append(QLine(grid_rect.left(), y, grid_rect.right(), y))
            else:
                dark_lines.append(QLine(grid_rect.left(), y, grid_rect.right(), y))

        return light_lines, dark_lines


GraphicsScenePainterFactory = providers.Factory(GraphicsScenePainter)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

"""A Deep Learning, node-based framework"""

from . import event_filters, main_window, node_editor, utils, widgets

__all__ = ["event_filters", "main_window", "node_editor", "utils", "widgets"]



from .panning_event_filter import PanningEventFilter
from .resizable_node_event_filter import ResizableNodeEventFilter
from .zoom_event_filter import ZoomEventFilter

"""This package includes some common event filters: Classes that implements new sets of
movements and operations on an object.

For example, by installing a `ZoomEventFilter` object on a QGrapQGraphicsView, you will
be able to zoom in/out the view using the mouse.
"""

__all__ = ["PanningEventFilter", "ZoomEventFilter", "ResizableNodeEventFilter"]



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from .project_gui import ProjectGUI, ProjectGUIFactory
from .project_manager_gui import ProjectManagerGUI, ProjectManagerGUISingleton

__all__ = [
    "ProjectGUI",
    "ProjectGUIFactory",
    "ProjectManagerGUI",
    "ProjectManagerGUISingleton",
]



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from enum import Enum
from typing import TYPE_CHECKING

from PySide2.QtCore import QEvent, QObject, Qt
from PySide2.QtGui import QGuiApplication, QMouseEvent

if TYPE_CHECKING:
    from PySide2.QtWidgets import QGraphicsView


class PanningEventFilter(QObject):
    """The PanningEventFilter class provides an easy implementation of a panning
    movement for a QGraphicsView object.

    Important:
        This filter only works with a QGraphicsView object.

    Attributes:
        state: The state of the movement (Panning or not).
        button_used_for_panning: The button that must be pressed and dragged for panning
        the view.

    Examples:
        view = QGraphicsView()
        panning_event_filter = PanningEventFilter()

        view.installEventFilter(panning_event_filter)

        # Now the `view` can be panned using the mouse.
    """

    class State(Enum):
        Idle = 0
        Panning = 1

    def __init__(self, parent: "QObject" = None):
        super().__init__(parent)

        self.__state = self.State.Idle

        self.__panning_start_x = 0
        self.__panning_start_y = 0

        self.button_used_for_panning = Qt.MiddleButton

    def is_panning(self) -> bool:
        """Checks if the view is currently being panned."""
        return self.__state == self.State.Panning

    def eventFilter(self, obj: "QObject", event: "QEvent") -> bool:
        """Intercepts events emitted by `obj`. Implements a panning effect when the mouse
        is dragged throughout the scene.

        This filter only works with a QGraphicsView object.

        Args:
            obj: The object (QGraphicsView) to pan.
            event: The generated event.

        Returns:
            If the event should be filtered out or not (Stopping it being handled by
            other objects).
        """
        if self.__panning_button_clicked(event):
            self.__start_panning_view(obj, event)
            return True

        if event.type() == QEvent.MouseMove and self.is_panning():
            self.__pan_view(obj, event)
            return True

        if self.__panning_button_released(event):
            self.__stop_panning_view(obj, event)
            return True

        return super().eventFilter(obj, event)

    def __panning_button_clicked(self, event: "QMouseEvent") -> bool:
        """Checks if the button defined as the "panning button" was clicked.

        This starts the panning movement.
        """
        return (
            event.type() == QMouseEvent.MouseButtonPress
            and event.button() == self.button_used_for_panning
        )

    def __panning_button_released(self, event: "QMouseEvent") -> bool:
        """Checks if the button defined as the "panning button" was released.

        This ends the panning movement.
        """
        return (
            event.type() == QMouseEvent.MouseButtonRelease
            and event.button() == self.button_used_for_panning
        )

    def __start_panning_view(self, view: "QGraphicsView", event: "QMouseEvent"):
        """Responds to the event of start panning the view.

        Changes the mouse icon to a dragging hand, and saves the last clicked position.
        This is used for calculating the mouse displacement.

        Args:
            view: The QGraphicsView object to pan.
            event: Mouse event.
        """
        QGuiApplication.setOverrideCursor(Qt.ClosedHandCursor)

        self.__panning_start_x = event.x()
        self.__panning_start_y = event.y()

        self.__state = self.State.Panning

    def __pan_view(self, view: "QGraphicsView", event: "QMouseEvent"):
        """Pans the view using the mouse movement.

        The scrollbars are moved along the view (They're disabled by default, but if
        they were enabled, the scrollbars would be scrolled along the view
        displacement).

        Args:
            view: The QGraphicsView object to pan.
            event: Generated mouse event.
        """
        # Move view by using the scrollbars
        view.horizontalScrollBar().setValue(
            view.horizontalScrollBar().value() - (event.x() - self.__panning_start_x)
        )

        view.verticalScrollBar().setValue(
            view.verticalScrollBar().value() - (event.y() - self.__panning_start_y)
        )

        # Set new "last panning values"
        self.__panning_start_x = event.x()
        self.__panning_start_y = event.y()

    def __stop_panning_view(self, view: "QGraphicsView", event: "QMouseEvent"):
        """Responds to the event of start dragging the view for panning it.

        Changes the mouse icon back to the default icon.

        Args:
            view: The QGraphicsView object to pan.
            event: Mouse event.
        """
        QGuiApplication.restoreOverrideCursor()

        self.__state = self.State.Idle



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING, Any, Dict

import dependency_injector.providers as providers
from PySide2.QtCore import Qt, Signal
from PySide2.QtWidgets import (
    QDialog,
    QGraphicsItem,
    QGraphicsObject,
    QGraphicsProxyWidget,
    QPushButton,
    QVBoxLayout,
    QWidget,
)

from dial_gui.event_filters import ResizableNodeEventFilter

from .graphics_node_painter import GraphicsNodePainterFactory
from .graphics_port import GraphicsPortFactory
from .graphics_port_painter import (
    InputGraphicsPortPainterFactory,
    OutputGraphicsPortPainterFactory,
)

if TYPE_CHECKING:
    from PySide2.QtCore import QRectF
    from PySide2.QtGui import QPainter, QMouseEvent

    from dial_core.node_editor import Node, Port  # noqa: F401
    from PySide2.QtWidgets import QStyleOptionGraphicsItem, QGraphicsSceneMouseEvent
    from dial_gui.node_editor import GraphicsPort  # noqa: F401
    from .graphics_scene import GraphicsScene


class GraphicsNode(QGraphicsObject):
    class ProxyWidget(QGraphicsProxyWidget):
        widget_resized = Signal("QSizeF")

        def resize(self, x_or_point, y=None):
            if isinstance(x_or_point, float):
                super().resize(x_or_point, y)
            else:
                super().resize(x_or_point)

            self.widget_resized.emit(self.size())

    def __init__(
        self,
        node: "Node",
        painter_factory: "providers.Factory",
        graphics_scene: "GraphicsScene" = None,
        parent: "QGraphicsItem" = None,
    ):
        super().__init__(parent)

        # Components
        self._node = node
        self._node.graphics_node = self  # type: ignore
        self.__graphics_scene = graphics_scene

        # GraphicsPorts
        self._input_graphics_ports = self.__create_graphics_ports(
            self._node.inputs, InputGraphicsPortPainterFactory
        )
        self._output_graphics_ports = self.__create_graphics_ports(
            self._node.outputs, OutputGraphicsPortPainterFactory
        )
        # Proxy
        self._proxy_widget = self.ProxyWidget(parent=self)
        self._proxy_widget.setWidget(
            self._node.inner_widget if self._node.inner_widget else QWidget()
        )

        # Painter
        self._painter_factory = painter_factory
        self._graphics_node_painter = painter_factory(graphics_node=self)

        # Flags
        self.setFlag(QGraphicsItem.ItemIsSelectable)
        self.setFlag(QGraphicsItem.ItemIsMovable)
        self.setAcceptHoverEvents(True)

        # Filters
        self.__resizable_node_event_filter = ResizableNodeEventFilter(parent=self)
        self.installEventFilter(self.__resizable_node_event_filter)

        # Connections
        self._proxy_widget.widget_resized.connect(
            lambda _: self._graphics_node_painter.recalculateGeometry()
        )

    @property
    def title(self):
        return self._node.title

    @property
    def painter(self):
        return self._graphics_node_painter

    @property
    def inputs(self):
        return self._input_graphics_ports

    @property
    def outputs(self):
        return self._output_graphics_ports

    def boundingRect(self) -> "QRectF":
        """Returns a rect enclosing the node."""
        return self._graphics_node_painter.boundingRect()

    def itemChange(self, change: "QGraphicsItem.GraphicsItemChange", value: Any) -> Any:
        if change == self.ItemSelectedChange:
            self._graphics_node_painter.itemChange(change, value)

            # Selected items gets a high Z value, so they're displayed on top of other
            # nodes. When unselected, return back to a low Z value.
            if value:
                self.setZValue(10)
            else:
                self.setZValue(0)

            return value

        return super().itemChange(change, value)

    def mouseDoubleClickEvent(self, event: "QGraphicsSceneMouseEvent"):
        if event.button() == Qt.LeftButton:
            self.__toggle_widget_dialog(event)

    def __toggle_widget_dialog(self, event: "QMouseEvent"):
        """Shows the Node `inner_widget` on a new dialog. The content of the node is
        substituted with a button that hides the dialog and shows the inner_widget back
        in the node when pressed.
        """

        node_inner_widget = self._proxy_widget.widget()
        previous_node_size = node_inner_widget.size()

        show_here_button = QPushButton("Show here")
        show_here_button.setMinimumSize(200, 100)

        # Replace the node widget with the button
        self.__set_inner_widget(show_here_button)

        # Create a new dialog for displaying the node widget
        dialog = QDialog()
        dialog.setWindowTitle(self._node.title)

        layout = QVBoxLayout()
        layout.addWidget(node_inner_widget)
        dialog.setLayout(layout)

        dialog.show()

        def place_widget_back_in_node():
            # Widgets embedded in nodes can't have parents
            node_inner_widget.setParent(None)

            node_inner_widget.resize(previous_node_size)
            self.__set_inner_widget(node_inner_widget)

            dialog.close()

        # The widget will be displayed back in the node when the dialog is closed or
        # when the "show here" button is pressed
        dialog.finished.connect(place_widget_back_in_node)
        show_here_button.clicked.connect(place_widget_back_in_node)

    def __set_inner_widget(self, widget: "QWidget"):
        """Sets a new widget inside the node."""
        self.prepareGeometryChange()

        self._proxy_widget.setWidget(widget)

        self._graphics_node_painter.repositionWidget()
        self._graphics_node_painter.recalculateGeometry()

    def __create_graphics_ports(
        self, ports_dict: Dict[str, "Port"], painter_factory: "providers.Factory"
    ) -> Dict[str, "GraphicsPort"]:
        """Creates new GraphicsPort items from regular Port objects."""
        graphics_ports_dict = {}

        for name in ports_dict.keys():
            port = ports_dict[name]

            try:
                port.graphics_port.painter_factory = painter_factory
                port.graphics_port.set_parent_graphics_node(self)
                graphics_ports_dict[name] = port.graphics_port

            except AttributeError:
                graphics_ports_dict[name] = GraphicsPortFactory(
                    port=port, painter_factory=painter_factory, parent=self
                )

        return graphics_ports_dict

    def __getstate__(self):
        return {"pos": self.pos(), "proxy_size": self._proxy_widget.size()}

    def __setstate__(self, new_state: dict):
        self.prepareGeometryChange()
        self.setPos(new_state["pos"])

        self._proxy_widget.resize(new_state["proxy_size"])

    def __reduce__(self):
        return (GraphicsNode, (self._node, self._painter_factory), self.__getstate__())

    def paint(
        self, painter: "QPainter", option: "QStyleOptionGraphicsItem", widget: "QWidget"
    ):
        """Paints the GraphicsNode item."""
        self._graphics_node_painter.paint(painter, option, widget)


GraphicsNodeFactory = providers.Factory(
    GraphicsNode, painter_factory=GraphicsNodePainterFactory.delegate()
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING, Any, Optional, Union

from dial_core.node_editor import Node
from dial_core.utils import log
from dial_gui.event_filters import PanningEventFilter, ZoomEventFilter
from dial_gui.node_editor import (
    GraphicsConnection,
    GraphicsConnectionFactory,
    GraphicsNode,
    GraphicsNodeFactory,
    GraphicsPort,
    GraphicsPortPainter,
)
from dial_gui.widgets.menus import NodesMenuFactory
from PySide2.QtCore import Qt
from PySide2.QtGui import QCursor, QPainter
from PySide2.QtWidgets import QGraphicsView

from .node_editor_view_menu import NodeEditorViewMenuFactory

if TYPE_CHECKING:
    from PySide2.QtGui import QContextMenuEvent
    from PySide2.QtCore import QObject
    from PySide2.QtGui import QMouseEvent, QWheelEvent
    from PySide2.QtWidgets import QTabWidget, QWidget


class NodeEditorView(QGraphicsView):
    def __init__(self, tabs_widget: "QTabWidget", parent: "QWidget" = None):
        super().__init__(parent)

        self.__tabs_widget = tabs_widget

        self.__new_connection: Optional["GraphicsConnection"] = None

        self.__panning_event_filter = PanningEventFilter(parent=self)
        self.__zoom_event_filter = ZoomEventFilter(parent=self)
        self.installEventFilter(self.__zoom_event_filter)

        self.setRenderHints(
            QPainter.Antialiasing
            | QPainter.HighQualityAntialiasing
            | QPainter.TextAntialiasing
            | QPainter.SmoothPixmapTransform
        )

        self.setDragMode(QGraphicsView.RubberBandDrag)

        # View actions
        self.set_panning(True)
        self.set_zooming(True)

        # Hide scrollbars
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        # Set anchor under mouse (for zooming)
        self.setTransformationAnchor(QGraphicsView.AnchorUnderMouse)

    def set_panning(self, toggle: bool):
        """Toggles if the view can be panned or not with the mouse."""
        self.__toggle_event_filter(toggle, self.__panning_event_filter)

    def set_zooming(self, toggle: bool):
        """Toggles if the view can be zoomed or not with the mouse wheel."""
        self.__toggle_event_filter(toggle, self.__zoom_event_filter)

    def mousePressEvent(self, event: "QMouseEvent"):
        # TODO: Explain why
        # event.ignore()
        if event.button() == self.__panning_event_filter.button_used_for_panning:
            event.ignore()
            return

        if event.button() == Qt.LeftButton:
            self.__start_dragging_connection(event)
            return

        super().mousePressEvent(event)

    def mouseMoveEvent(self, event: "QMouseEvent"):
        # TODO: Explain why
        # event.ignore()
        if self.__panning_event_filter.is_panning():
            event.ignore()
            return

        if self.__is_dragging_connection():
            self.__dragging_connection(event)
            return

        super().mouseMoveEvent(event)

    def mouseReleaseEvent(self, event: "QMouseEvent"):
        # TODO: Explain why
        if self.__panning_event_filter.is_panning():
            event.ignore()
            return

        if event.button() == Qt.LeftButton:
            self.__stop_dragging_connection(event)

        super().mouseReleaseEvent(event)

    def wheelEvent(self, event: "QWheelEvent"):
        # TODO: Explain why
        event.ignore()

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Delete:
            # Remove selected elements
            for selected_item in self.scene().selectedItems():
                self.scene().removeItem(selected_item)

            self.scene().update()
            return

        super().keyPressEvent(event)

    def contextMenuEvent(self, event: "QContextMenuEvent"):
        item = self.__item_clicked_on(event)
        print(self.scene().selectedItems())

        if self.scene().selectedItems():
            context_menu = NodeEditorViewMenuFactory(node_editor_view=self, parent=self)
            context_menu.popup(event.globalPos())
            return

        if item is None:
            context_menu = NodesMenuFactory(parent=self)
            context_menu.node_created.connect(self.__create_graphics_node_from)
            context_menu.popup(event.globalPos())
            return

    def __create_graphics_node_from(self, node: "Node") -> "GraphicsNode":
        graphics_node = GraphicsNodeFactory(node)

        global_pos = self.mapFromGlobal(QCursor.pos())
        graphics_node.setPos(self.mapToScene(global_pos))

        self.scene().addItem(graphics_node)

        return graphics_node

    def __start_dragging_connection(self, event: "QMouseEvent"):
        """Starts creating a new connection by dragging the mouse.

        Only works when the user clicks on a GraphicsPort item.

        Args:
            event: Mouse event.
        """
        item: "GraphicsPort" = self.__item_clicked_on(event)

        if isinstance(item, GraphicsConnection):
            distance_to_start = (
                item.start_graphics_port.pos() - self.mapToScene(event.pos())
            ).manhattanLength()
            distance_to_end = (
                item.end_graphics_port.pos() - self.mapToScene(event.pos())
            ).manhattanLength()

            if distance_to_start < distance_to_end:
                end_graphics_port = item.end_graphics_port
                item.end = self.mapToScene(event.pos())
                item.start_graphics_port = end_graphics_port
            else:
                item.end = self.mapToScene(event.pos())

            GraphicsPortPainter.drawing_state = GraphicsPortPainter.DrawingState.Target
            GraphicsPortPainter.target_port_type = item.start_graphics_port.port_type
            self.scene().update()

            self.__new_connection = item
            return

        elif isinstance(item, GraphicsPort):
            log.get_logger(__name__).debug("Start dragging")

            self.__new_connection = self.__create_new_connection()
            self.__new_connection.start_graphics_port = item
            self.__new_connection.end = self.mapToScene(event.pos())

            GraphicsPortPainter.drawing_state = GraphicsPortPainter.DrawingState.Target
            GraphicsPortPainter.target_port_type = item.port_type
            self.scene().update()
            return

        super().mousePressEvent(event)

    def __stop_dragging_connection(self, event: "QMouseEvent"):
        """Stops dragging the connection.

        If the connection doesn't end on a GraphicsPort, the connection item is removed
        from the scene.

        Args:
            event: Mouse event.
        """
        if not self.__is_dragging_connection() or not self.__new_connection:
            super().mouseReleaseEvent(event)
            return

        item = self.__item_clicked_on(event)

        # The conection must end on a COMPATIBLE GraphicsPort item
        if isinstance(item, GraphicsPort) and item.is_compatible_with(
            self.__new_connection.start_graphics_port  # type: ignore
        ):
            start_graphics_port = self.__new_connection.start_graphics_port
            if not start_graphics_port._port.allows_multiple_connections:
                for connection in start_graphics_port.graphics_connections:
                    self.scene().removeItem(connection)

            if not item._port.allows_multiple_connections:
                for connection in item.graphics_connections:
                    self.scene().removeItem(connection)

            self.__new_connection.start_graphics_port = start_graphics_port
            self.__new_connection.end_graphics_port = item
            self.scene().addItem(self.__new_connection)
        else:
            self.scene().removeItem(self.__new_connection)

        # Reset the connection item
        self.__new_connection = None

        # Reset the port painter
        GraphicsPortPainter.drawing_state = GraphicsPortPainter.DrawingState.Normal
        GraphicsPortPainter.target_port_type = None
        self.scene().update()

        super().mouseReleaseEvent(event)

    def __dragging_connection(self, event: "QMouseEvent"):
        """Drags a connection while the mouse is moving.

        Args:
            event: Mouse event.
        """
        if not self.__new_connection:
            super().mouseMoveEvent(event)
            return

        self.__new_connection.end = self.mapToScene(event.pos())

        item = self.__item_clicked_on(event)
        if isinstance(item, GraphicsPort):
            self.__new_connection.end = item.pos()

        super().mouseMoveEvent(event)

    def __is_dragging_connection(self) -> bool:
        """Checks if the user is currently dragging a connection or not."""
        return self.__new_connection is not None

    def __item_clicked_on(self, event: "QMouseEvent") -> Union["GraphicsPort", Any]:
        """Returns the graphical item under the mouse."""
        return self.itemAt(event.pos())

    def __create_new_connection(self) -> "GraphicsConnection":
        """Create a new connection on the scene."""
        connection = GraphicsConnectionFactory()
        self.scene().addItem(connection)

        return connection

    def __toggle_event_filter(self, toggle: bool, event_filter: "QObject"):
        """Toggles (Installs/Uninstalls) the specified event filter on this object."""
        if toggle:
            self.installEventFilter(event_filter)
        else:
            self.uninstallEventFilter(event_filter)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

import dependency_injector.providers as providers

from typing import TYPE_CHECKING

from PySide2.QtWidgets import QMenu, QAction

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget
    from .node_editor_view import NodeEditorView


class NodeEditorViewMenu(QMenu):
    def __init__(self, node_editor_view: "NodeEditorView", parent: "QWidget" = None):
        super().__init__(parent)

        self.__node_editor_view = node_editor_view

        self._remove_elements = QAction("Remove selected elements")
        self._remove_elements.triggered.connect(self.__remove_selected_elements)

        self.addAction(self._remove_elements)

    def __remove_selected_elements(self):
        for selected_item in self.__node_editor_view.scene().selectedItems():
            self.__node_editor_view.scene().removeItem(selected_item)

        self.__node_editor_view.scene().update()


NodeEditorViewMenuFactory = providers.Factory(NodeEditorViewMenu)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

"""
Starting point for the application GUI.
"""

from datetime import datetime
from typing import TYPE_CHECKING

from dial_core.utils import log
from PySide2.QtWidgets import QApplication

from dial_gui.main_window import MainWindowFactory

if TYPE_CHECKING:
    import argparse

LOGGER = log.get_logger(__name__)


def run(args: "argparse.Namespace"):
    """
    Show the main window and start the application.

    Warning:
        The system must be initialized before calling this function.

    See Also:
        `dial_core.initialization.initialize_application`
    """

    main_window = MainWindowFactory()
    main_window.show()

    LOGGER.debug("Command Line Arguments: %s", args)
    LOGGER.info("Dial.")
    LOGGER.info("Started on %s", datetime.now().ctime())

    return QApplication.exec_()



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

import dependency_injector.providers as providers

import logging
from typing import TYPE_CHECKING

from PySide2.QtCore import QSize
from PySide2.QtWidgets import QDialog, QVBoxLayout

from .logger_textbox import LoggerTextboxWidget, LoggerTextboxFactory

from dial_core.utils import log

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget


class LoggerDialog(QDialog):
    """The LoggerDialog class provides a dialog window prepared for displaying messages
    from the Python logging system.

    Examples:
        logger_dialog = LoggerDialog()
        logging.getLogger().addHandler(logger_dialog.handler)
    """

    def __init__(self, textbox_widget: "LoggerTextboxWidget", parent: "QWidget" = None):
        super().__init__(parent)

        self.textbox = textbox_widget
        self.textbox.setParent(self)

        self.setWindowTitle("Logging window")

        layout = QVBoxLayout()
        layout.addWidget(self.textbox)
        layout.setContentsMargins(0, 0, 0, 0)

        self.setLayout(layout)

        log.add_handler_to_root(self.handler())
        # Write on the window all the previous log messages
        self.textbox.set_plain_text(log.LOG_STREAM.getvalue())

    def handler(self) -> "logging.Handler":
        """Returns a logging handler associated to this dialog. Must be used to send log
        messages to it.
        """
        return self.textbox

    def sizeHint(self) -> "QSize":
        """Preferred size of this dialog."""
        return QSize(800, 600)


LoggerDialogFactory = providers.Factory(
    LoggerDialog, textbox_widget=LoggerTextboxFactory
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

"""
Utility methods to work with Tkinter.

Used for showing window messages if we can't access PySide2 (Qt), as Tkinter is
guaranteed to be installed along Python3.
"""

import tkinter as tk
from tkinter import messagebox


def showerror(message: str, title: str = "Error"):
    """
    Shows an error window with a custom message.
    """

    root = tk.Tk()
    root.withdraw()

    messagebox.showerror(title, message)



#!/usr/bin/env python3
# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

"""
Entry point for dial ui.
"""

import sys
from typing import List


def main(sys_args: List = sys.argv[1:]):
    """
    Entry point for Dial. Initialize components and stars the application.

    Args:
        sys_args: A list of arguments from the command line.
    """
    import dial_core

    # Parse arguments
    app_config = dial_core.utils.initialization.parse_args(sys_args)

    # Initialize
    from dial_gui.utils import initialization

    initialization.initialize(app_config)

    from dial_gui import app

    # Run
    sys.exit(app.run(app_config))


if __name__ == "__main__":
    main()



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from enum import Enum, Flag, auto
from typing import TYPE_CHECKING

from PySide2.QtCore import QEvent, QObject, Qt

if TYPE_CHECKING:
    from dial_gui.node_editor import GraphicsNode


class ResizableNodeEventFilter(QObject):
    """The ResizableNodeEventFilter class provides resizing anchors for a Graphicsnode
    object.

    The anchors can be dragged freely using the mouse, and the node will be resized
    accordingly.

    Important:
        This filter only works with a GraphicsNode object.

    Attributes:
        resize_margins: Size of the margins that can be clicked to start the resizing
        event.
        button_used_for_resizing: The button that mast be pressed and dragged for
        resizing the node.

    Examples:
        node = GraphicsNode()
        resizable_node_event_filter = ResizableNodeEventFilter()

        node.installEventFilter(resizable_node_event_filter)
    """

    class State(Enum):
        Idle = 0
        Resizing = 1

    class MarginClicked(Flag):
        NoMargin = 0
        Top = auto()
        Bottom = auto()
        Left = auto()
        Right = auto()

    def __init__(self, parent: "QObject" = None):
        super().__init__(parent)

        self.__state = self.State.Idle
        self.__margins_clicked = self.MarginClicked.NoMargin

        self.resize_margins = 12

        self.button_used_for_resizing = Qt.LeftButton

    @property
    def state(self) -> "ResizableNodeEventFilter.State":
        """Returns the state of the event filter (Normal, Resizing...)"""
        return self.__state

    def is_resizing(self) -> bool:
        """Checks if the event filter is currently resizing an object."""
        return self.__state == self.State.Resizing

    def eventFilter(self, node: "GraphicsNode", event: "QEvent") -> bool:
        """Tracks the mouse movements to do the actual resizing.

        When the mouse hovers over the node, the cursor icon is changed to reflect the
        direction of the resizing (P.E: If the mouse is on the left side of the node,
        a <-> icon will appear).

        When the resizing button is clicked, the node will be resized until the button
        is released.
        """
        if event.type() == QEvent.GraphicsSceneHoverMove:
            self.__track_margins_under_cursor(node, event)
            return True

        if self.__resize_button_clicked(event) and self.__is_inside_resize_margins(
            node, event
        ):
            self.__start_resizing_node(node, event)
            return True

        if self.is_resizing():
            if event.type() == QEvent.GraphicsSceneMouseMove:
                self.__resizing_node(node, event)
                return True

            if self.__resize_button_released(event):
                self.__stop_resizing_node(node, event)
                return True

        return super().eventFilter(node, event)

    def __resize_button_clicked(self, event: "QEvent") -> bool:
        """Checks if the button designed for resizing was pressed."""
        return (
            event.type() == QEvent.GraphicsSceneMousePress
            and event.button() == self.button_used_for_resizing
        )

    def __resize_button_released(self, event: "QEvent") -> bool:
        """Checks if the button designed for resizing was released."""
        return (
            event.type() == QEvent.GraphicsSceneMouseRelease
            and event.button() == self.button_used_for_resizing
        )

    def __start_resizing_node(self, node: "GraphicsNode", event: "QEvent"):
        """Starts resizing the node.

        Saves some positions for reference during the resizing event.
        """
        self.__state = self.State.Resizing

        # Saves some information of the cursor and the node prior to resizing
        self.__initial_resize_pos = event.scenePos()
        self.__initial_node_pos = node.pos()
        self.__initial_node_size = node._proxy_widget.size()

    def __resizing_node(self, node: "GraphicsNode", event: "QEvent"):
        """Resizes the node while dragging one of the margins. Calculates the resulting
        size and position of the node and applies it."""
        diff = event.scenePos() - self.__initial_resize_pos

        new_x = 0
        new_y = 0
        new_w = 0
        new_h = 0

        if self.__margins_clicked & self.MarginClicked.Left:
            new_x = (
                diff.x()
                if node._proxy_widget.size().width()
                > node._proxy_widget.minimumSize().width()
                else self.__initial_node_size.width()
                - node._proxy_widget.minimumSize().width()
            )
            new_w = -diff.x()

        elif self.__margins_clicked & self.MarginClicked.Right:
            new_w = diff.x()

        if self.__margins_clicked & self.MarginClicked.Top:
            new_y = (
                diff.y()
                if node._proxy_widget.size().height()
                > node._proxy_widget.minimumSize().height()
                else self.__initial_node_size.height()
                - node._proxy_widget.minimumSize().height()
            )
            new_h = -diff.y()

        elif self.__margins_clicked & self.MarginClicked.Bottom:
            new_h = diff.y()

        node.prepareGeometryChange()

        node._proxy_widget.resize(
            self.__initial_node_size.width() + new_w,
            self.__initial_node_size.height() + new_h,
        )

        node.setPos(
            self.__initial_node_pos.x() + new_x, self.__initial_node_pos.y() + new_y
        )

    def __stop_resizing_node(self, node: "GraphicsNode", event: "QEvent"):
        """Stops resizing the node."""
        self.__state = self.State.Idle

    def __is_inside_resize_margins(self, node: "GraphicsNode", event: "QEvent") -> bool:
        """Checks if the cursor is currently on the margins of the node."""
        return self.__margins_clicked != self.MarginClicked.NoMargin

    def __track_margins_under_cursor(self, node: "GraphicsNode", event: "QEvent"):
        """Sets different flags marking on top of which margin the cursor is."""
        x_pos = event.pos().x()
        y_pos = event.pos().y()

        # Horizontal margins
        if x_pos <= self.resize_margins:
            self.__margins_clicked |= self.MarginClicked.Left
            node.setCursor(Qt.SizeHorCursor)
        elif node.boundingRect().width() - x_pos <= self.resize_margins:
            self.__margins_clicked |= self.MarginClicked.Right
            node.setCursor(Qt.SizeHorCursor)
        else:
            self.__margins_clicked &= ~self.MarginClicked.Left
            self.__margins_clicked &= ~self.MarginClicked.Right

        # Vertical margins
        if y_pos <= self.resize_margins:
            self.__margins_clicked |= self.MarginClicked.Top
            node.setCursor(Qt.SizeVerCursor)
        elif node.boundingRect().height() - y_pos <= self.resize_margins:
            self.__margins_clicked |= self.MarginClicked.Bottom
            node.setCursor(Qt.SizeVerCursor)
        else:
            self.__margins_clicked &= ~self.MarginClicked.Top
            self.__margins_clicked &= ~self.MarginClicked.Bottom

        if self.__margins_clicked == (self.MarginClicked.Left | self.MarginClicked.Top):
            node.setCursor(Qt.SizeFDiagCursor)

        if self.__margins_clicked == (
            self.MarginClicked.Left | self.MarginClicked.Bottom
        ):
            node.setCursor(Qt.SizeBDiagCursor)

        if self.__margins_clicked == (
            self.MarginClicked.Right | self.MarginClicked.Top
        ):
            node.setCursor(Qt.SizeBDiagCursor)

        if self.__margins_clicked == (
            self.MarginClicked.Right | self.MarginClicked.Bottom
        ):
            node.setCursor(Qt.SizeFDiagCursor)

        if self.__margins_clicked == self.MarginClicked.NoMargin:
            node.unsetCursor()



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from PySide2.QtWidgets import QVBoxLayout, QWidget

from .plugins_table_model import PluginsTableModelFactory
from .plugins_table_view import PluginsTableViewFactory

if TYPE_CHECKING:
    from .plugins_table_model import PluginsTableModel
    from .plugins_table_view import PluginsTableView


class PluginsTableWidget(QWidget):
    def __init__(
        self,
        plugins_table_model: "PluginsTableModel",
        plugins_table_view: "PluginsTableView",
        parent: "QWidget" = None,
    ):
        super().__init__(parent)

        self.__plugins_table_model = plugins_table_model
        self.__plugins_table_model.setParent(self)

        self.__plugins_table_view = plugins_table_view
        self.__plugins_table_view.setParent(self)

        self.__plugins_table_view.setModel(self.__plugins_table_model)

        self.__main_layout = QVBoxLayout()
        self.__main_layout.setContentsMargins(0, 0, 0, 0)

        self.__main_layout.addWidget(self.__plugins_table_view)

        self.setLayout(self.__main_layout)


PluginsTableWidgetFactory = providers.Factory(
    PluginsTableWidget,
    plugins_table_model=PluginsTableModelFactory,
    plugins_table_view=PluginsTableViewFactory,
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from PySide2.QtGui import QColor
from tensorflow.keras import Model

from dial_core.datasets import Dataset
from dial_core.utils import Dial


class TypeColor:
    colors = {
        int: QColor("#B54747"),
        str: QColor("#0056A6"),
        Dataset: QColor("#6666FF"),
        Dial.KerasLayerListMIME: QColor("#AA0000"),
        Model: QColor("#0000AA"),
    }

    @classmethod
    def get_color_for(cls, port_type):
        try:
            return cls.colors[port_type]
        except KeyError:
            return QColor("#000000")



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from dial_core.project import Project
from dial_gui.node_editor import GraphicsSceneFactory

if TYPE_CHECKING:
    from dial_gui.node_editor import GraphicsScene


class ProjectGUI(Project):
    def __init__(self, name: str, graphics_scene: "GraphicsScene"):
        super().__init__(name, graphics_scene.scene)

        self.__graphics_scene = graphics_scene

    @property
    def graphics_scene(self):
        return self.__graphics_scene

    def __reduce__(self):
        return (ProjectGUI, (self.name, self.__graphics_scene))


ProjectGUIFactory = providers.Factory(
    ProjectGUI, name="Default Project", graphics_scene=GraphicsSceneFactory
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

"""
Utility and helper methods (Logging system, version checkers, code timers...).
"""

from . import application, initialization, tkinter

__all__ = ["initialization", "tkinter", "application"]



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from enum import IntEnum
from typing import TYPE_CHECKING, Optional, Any

import dependency_injector.providers as providers
from PySide2.QtCore import QAbstractTableModel, QModelIndex, QSize, Qt
from dial_gui.utils import application

from dial_core.plugin import PluginManagerSingleton
from dial_core.utils import log

if TYPE_CHECKING:
    from PySide2.QtWidgets import QObject
    from dial_core.plugin import PluginManager, Plugin


class PluginsTableModel(QAbstractTableModel):
    class ColumnLabel(IntEnum):
        Active = 0
        Name = 1
        Summary = 2
        Version = 3

    def __init__(self, plugin_manager: "PluginManager", parent: "QObject" = None):
        super().__init__(parent)

        self.__plugin_manager = plugin_manager

        self.__role_map = {
            Qt.DisplayRole: self.__data_display_role,
            Qt.CheckStateRole: self.__data_checkstate_role,
            Qt.TextAlignmentRole: self.__data_textalignment_role,
            Qt.SizeHintRole: self.__data_sizehint_role,
        }

    def rowCount(self, parent=QModelIndex()):
        return len(self.__plugin_manager.installed_plugins)

    def columnCount(self, parent=QModelIndex()):
        return len(self.ColumnLabel)

    def index(self, row: int, column: int, parent: "QModelIndex") -> "QModelIndex":
        if not self.hasIndex(row, column, parent):
            return QModelIndex()

        return self.createIndex(
            row, column, list(self.__plugin_manager.installed_plugins.values())[row]
        )

    def flags(self, index: "QModelIndex") -> int:
        general_flags = super().flags(index)

        if index.column() == self.ColumnLabel.Active:
            return general_flags | Qt.ItemIsUserCheckable

        return general_flags

    def headerData(
        self, section: int, orientation: "Qt.Orientation", role=Qt.DisplayRole
    ) -> Optional[str]:

        if role != Qt.DisplayRole:
            return None

        if orientation == Qt.Horizontal:
            return self.ColumnLabel(section).name

        return None

    def data(self, index: "QModelIndex", role=Qt.DisplayRole):
        if role in self.__role_map:
            return self.__role_map[role](index)

        return None

    def setData(
        self, index: "QModelIndex", value: Any, role: int = Qt.EditRole
    ) -> bool:
        if not index.isValid():
            return False

        plugin: "Plugin" = index.internalPointer()

        if role == Qt.CheckStateRole:
            if index.column() == self.ColumnLabel.Active:
                plugin.active = bool(value)

                log.get_logger("plugins").info(
                    f'Plugin "{plugin.name}" has been '
                    f"{'activated' if plugin.active else 'deactivated'}."
                )

        return True

    def __data_display_role(self, index: "QModelIndex") -> Optional["str"]:
        if not index.isValid():
            return None

        plugin: "Plugin" = index.internalPointer()

        if index.column() == self.ColumnLabel.Name:
            return plugin.name

        if index.column() == self.ColumnLabel.Summary:
            return plugin.summary

        if index.column() == self.ColumnLabel.Version:
            return plugin.version

        return None

    def __data_checkstate_role(self, index: "QModelIndex") -> Optional["Qt.CheckState"]:
        if not index.isValid():
            return None

        plugin: "Plugin" = index.internalPointer()

        if index.flags() & Qt.ItemIsUserCheckable:
            return Qt.Checked if plugin.active else Qt.Unchecked

        return None

    def __data_textalignment_role(self, index: "QModelIndex"):
        if not index.isValid():
            return None

        if index.column() == self.ColumnLabel.Version:
            return Qt.AlignCenter

        return None

    def __data_sizehint_role(self, index: "QModelIndex"):
        if not index.isValid():
            return None

        if index.column() == self.ColumnLabel.Active:
            return QSize(0, 0)

        return None


PluginsTableModelFactory = providers.Factory(
    PluginsTableModel,
    plugin_manager=PluginManagerSingleton(
        installed_plugins_dict=application.installed_plugins_file_content()
    ),
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:


from .graphics_connection import GraphicsConnection, GraphicsConnectionFactory
from .graphics_connection_painter import (
    GraphicsConnectionPainter,
    GraphicsConnectionPainterFactory,
)
from .graphics_node import GraphicsNode, GraphicsNodeFactory
from .graphics_port import GraphicsPort
from .graphics_port_painter import GraphicsPortPainter
from .graphics_scene import GraphicsScene, GraphicsSceneFactory
from .node_editor_view import NodeEditorView
from .node_editor_window import NodeEditorWindow

__all__ = [
    "GraphicsNode",
    "GraphicsNodeFactory",
    "GraphicsScene",
    "GraphicsConnection",
    "GraphicsConnectionFactory",
    "GraphicsConnectionPainter",
    "GraphicsConnectionPainterFactory",
    "NodeEditorWindow",
    "NodeEditorView",
    "GraphicsPort",
    "GraphicsPortPainter",
    "GraphicsSceneFactory",
]



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from enum import Enum
from typing import TYPE_CHECKING, Any, Dict, List, Optional

import dependency_injector.providers as providers
from PySide2.QtCore import QRectF
from PySide2.QtWidgets import QGraphicsItem

from dial_core.node_editor import Port

from .graphics_connection import GraphicsConnection, GraphicsConnectionFactory
from .graphics_port_painter import GraphicsPortPainterFactory

if TYPE_CHECKING:
    from .graphics_node import GraphicsNode
    from PySide2.QtGui import QPainter
    from PySide2.QtCore import QPointF
    from PySide2.QtWidgets import QWidget, QStyleOptionGraphicsItem


class GraphicsPort(QGraphicsItem):
    """Class representing a port for a node.

    Can be used as a start/end point for dragging connections between nodes.

    Attributes:
        graphics_node: Parent GraphicsNode object where this port is located.
        radius: Radius of the port (used for drawing).
        color: Color of the port.
        port: Port object associated with this GraphicsPort object.
        connections: GraphicsConnection objects connected to this port.
    """

    class DrawingState(Enum):
        Normal = 0
        Dragging = 1

    drawing_state = DrawingState.Normal
    drawing_type = None

    def __init__(
        self,
        port: "Port",
        painter_factory: "providers.Factory",
        parent: "GraphicsNode" = None,
    ):
        super().__init__(parent)

        self.radius = 8
        self.margin = 12

        self._port = port
        self._port.graphics_port = self  # type: ignore

        self.__graphics_node = parent
        self.__graphics_connections: List["GraphicsConnection"] = []

        self._painter_factory = painter_factory
        self._graphics_port_painter = painter_factory(graphics_port=self)

        self.__create_graphics_connections()

    @property
    def name(self):  # TODO: Eventually remove
        """Returns the name of the graphics port."""
        return self._port.name

    @property
    def port_type(self):
        return self._port.port_type

    def is_compatible_with(self, graphics_port: "GraphicsPort"):
        return self._port.is_compatible_with(graphics_port._port)

    @property
    def graphics_node(self) -> Optional["GraphicsNode"]:
        """Returns the parent GraphicsNode where this port is located."""
        return self.__graphics_node

    @property
    def graphics_connections(self) -> List["GraphicsConnection"]:
        """Returns a list of the GraphicsConnections item connected to this port."""
        return self.__graphics_connections

    @property
    def painter(self):
        return self._graphics_port_painter

    @property
    def painter_factory(self):
        return self._painter_factory

    @painter_factory.setter
    def painter_factory(self, painter_factory):
        self._painter_factory = painter_factory
        self._graphics_port_painter = painter_factory(graphics_port=self)

    def set_parent_graphics_node(self, graphics_node: "GraphicsNode"):
        super().setParentItem(graphics_node)

        self.__graphics_node = graphics_node

    def pos(self) -> "QPointF":
        """Returns the position of the GraphicsPort (In terms of scene coordinates)."""
        return (
            self.graphics_node.pos() + super().pos()
            if self.graphics_node
            else super().pos()
        )

    def add_connection(self, connection_item: "GraphicsConnection"):
        """Adds a new GraphicsConnection item to the list of connections.

        Args:
            connection_item: A GraphicsConnection object.
        """
        if connection_item.is_connected():
            connection_item.start_graphics_port._port.connect_to(  # type: ignore
                connection_item.end_graphics_port._port  # type:ignore
            )

        self.__graphics_connections.append(connection_item)

    def remove_connection(self, connection_item: "GraphicsConnection"):
        """Removes an existent GraphicsConnection item from the list of connections.

        Doesn't do anything if the item to remove is not present on the connections
        list.

        Args:
            connection_item: A GraphicsConnection object.
        """
        if connection_item.is_connected():
            connection_item.start_graphics_port._port.disconnect_from(  # type: ignore
                connection_item.end_graphics_port._port  # type: ignore
            )

        try:
            self.__graphics_connections.remove(connection_item)
        except ValueError:
            pass

    def boundingRect(self) -> "QRectF":
        """Returns an enclosing rect for the port, PLUS a margin. All the boundingRect()
        area is clickable by the user and can be used as a start/end zone for drag/drop
        connections.

        Important:
            Do not use this function for painting. The area for painting doesn't
            includes the margins, only the radius.
        """
        return QRectF(
            -self.radius - self.margin,
            -self.radius - self.margin,
            2 * self.radius + 2 * self.margin,
            2 * self.radius + 2 * self.margin,
        )

    def __create_graphics_connections(self):
        for connected_port in self._port.connections:
            if not hasattr(connected_port, "graphics_port"):
                setattr(
                    connected_port, "graphics_port", GraphicsPortFactory(connected_port)
                )

            graphics_ports = list(
                map(
                    lambda x: x.start_graphics_port,
                    connected_port.graphics_port.graphics_connections,
                )
            ) + list(
                map(
                    lambda x: x.end_graphics_port,
                    connected_port.graphics_port.graphics_connections,
                )
            )

            if self not in graphics_ports:
                connection_item = GraphicsConnectionFactory()
                connection_item.start_graphics_port = self
                connection_item.end_graphics_port = connected_port.graphics_port

    def __getstate__(self) -> Dict[str, Any]:
        return {"connections": self.__graphics_connections, "pos": self.pos()}

    def __setstate__(self, new_state: Dict[str, Any]):
        self.__graphics_connections = new_state["connections"]
        self.setPos(new_state["pos"])

    def __reduce__(self):
        return (
            GraphicsPort,
            (self._port, self._painter_factory, self.__graphics_node),
            self.__getstate__(),
        )

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self._port == other._port

    def paint(
        self,
        painter: "QPainter",
        option: "QStyleOptionGraphicsItem",
        widget: "QWidget" = None,
    ):
        """Paints the port."""
        self._graphics_port_painter.paint(painter, option, widget)


GraphicsPortFactory = providers.Factory(
    GraphicsPort, painter_factory=GraphicsPortPainterFactory.delegate()
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING, Any

import dependency_injector.providers as providers
from PySide2.QtCore import QRectF, Qt
from PySide2.QtGui import QBrush, QColor, QPainter, QPainterPath, QPen
from PySide2.QtWidgets import QGraphicsItem, QGraphicsTextItem

if TYPE_CHECKING:
    from .graphics_node import GraphicsNode
    from PySide2.QtWidgets import QStyleOptionGraphicsItem, QWidget


class GraphicsNodePainter:
    def __init__(self, graphics_node: "GraphicsNode"):
        self.__graphics_node = graphics_node

        # Dimensions
        self.padding = 12
        self.clickable_margin = 15
        self.round_edge_size = 10

        # Colors/Pens/Brushes
        self.__title_color = Qt.white
        self.__outline_selected_color = QColor("#FFA637")
        self.__outline_default_color = QColor("#000000")

        self.__outline_pen = QPen(self.__outline_default_color)
        self.__title_background_brush = QBrush(QColor("#FF313131"))
        self.__background_brush = QBrush(QColor("#E3212121"))

        # Graphics Components
        self.__graphics_title = QGraphicsTextItem(parent=graphics_node)
        self.__graphics_title.setDefaultTextColor(self.__title_color)
        self.__graphics_title.setPlainText(self.__graphics_node.title)
        self.__graphics_title.setPos(self.padding, 0)

        # Position ProxyWidget
        self.repositionWidget()

        # position GraphicsPort objects
        def position_graphics_ports(x_offset, graphics_ports_dict):
            for i, graphics_port in enumerate(graphics_ports_dict.values()):
                graphics_port.setPos(
                    x_offset, self.title_height() + (graphics_port.radius * 4) * (i + 1)
                )

        position_graphics_ports(0, self.__graphics_node.inputs)
        position_graphics_ports(
            self.boundingRect().width(), self.__graphics_node.outputs
        )

    @property
    def outline_selected_color(self) -> "QColor":
        return self.__outline_selected_color

    @outline_selected_color.setter
    def outline_selected_color(self, color: "QColor"):
        self.__outline_selected_color = color

    @property
    def outline_default_color(self) -> "QColor":
        return self.__outline_default_color

    @outline_default_color.setter
    def outline_default_color(self, color: "QColor"):
        self.__outline_default_color = color

    def boundingRect(self) -> "QRectF":
        proxy_rect = self.__graphics_node._proxy_widget.boundingRect()

        return proxy_rect.adjusted(
            0, 0, self.padding * 2, self.title_height() + self.padding * 2
        ).normalized()

    def repositionWidget(self):
        self.__graphics_node._proxy_widget.setPos(
            self.padding, self.title_height() + self.padding
        )

    def recalculateGeometry(self):
        for graphics_port in self.__graphics_node.outputs.values():
            graphics_port.setX(self.boundingRect().width())

    def itemChange(self, change: "QGraphicsItem.GraphicsItemChange", value: Any) -> Any:
        if change == QGraphicsItem.ItemSelectedChange:
            self.__outline_pen.setColor(
                self.__outline_selected_color if value else self.__outline_default_color
            )

        return value

    def title_height(self) -> int:
        """Returns the height of the title graphics item."""
        return self.__graphics_title.boundingRect().height()

    def paint(
        self, painter: "QPainter", option: "QStyleOptionGraphicsItem", widget: "QWidget"
    ):
        """Paints the GraphicsNode item."""

        self.__paint_background(painter)
        self.__paint_title_background(painter)
        self.__paint_outline(painter)

    def __paint_background(self, painter: "QPainter"):
        """Paints the background of the node. Plain color, no lines."""
        path_background = QPainterPath()
        path_background.addRoundedRect(
            self.__graphics_node.boundingRect(),
            self.round_edge_size,
            self.round_edge_size,
        )

        painter.setPen(Qt.NoPen)
        painter.setBrush(self.__background_brush)

        painter.drawPath(path_background.simplified())

    def __paint_title_background(self, painter: "QPainter"):
        """Paints a little background behind the title text, at the top of the node."""
        title_rect = self.__graphics_title.boundingRect()

        path_title_background = QPainterPath()
        path_title_background.setFillRule(Qt.WindingFill)
        path_title_background.addRoundedRect(
            0,
            0,
            self.__graphics_node.boundingRect().width(),
            title_rect.height(),
            self.round_edge_size,
            self.round_edge_size,
        )

        # (Drawing rects to hide the two botton round edges)
        path_title_background.addRect(
            0,
            title_rect.height() - self.round_edge_size,
            self.round_edge_size,
            self.round_edge_size,
        )

        path_title_background.addRect(
            self.__graphics_node.boundingRect().width() - self.round_edge_size,
            title_rect.height() - self.round_edge_size,
            self.round_edge_size,
            self.round_edge_size,
        )

        painter.setPen(Qt.NoPen)
        painter.setBrush(self.__title_background_brush)
        painter.drawPath(path_title_background.simplified())

    def __paint_outline(self, painter: "QPainter"):
        """Paints the outline of the node. Depending on if its selected or not, the
        color of the outline changes."""
        path_outline = QPainterPath()
        path_outline.addRoundedRect(
            self.__graphics_node.boundingRect(),
            self.round_edge_size,
            self.round_edge_size,
        )

        painter.setPen(self.__outline_pen)
        painter.setBrush(Qt.NoBrush)
        painter.drawPath(path_outline.simplified())


GraphicsNodePainterFactory = providers.Factory(GraphicsNodePainter)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from dial_gui.widgets.menus import (
    FileMenuFactory,
    PluginsMenuFactory,
    ProjectsMenuFactory,
    WindowsMenuFactory,
)
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QMenuBar

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget
    from dial_gui.widgets.menus import FileMenu, PluginsMenu, WindowsMenu, ProjectsMenu


class MainMenuBar(QMenuBar):
    """
    Top menu bar for the main window.
    """

    quit = Signal()

    def __init__(
        self,
        file_menu: "FileMenu",
        plugins_menu: "PluginsMenu",
        windows_menu: "WindowsMenu",
        projects_menu: "ProjectsMenu",
        parent: "QWidget" = None,
    ):
        super().__init__(parent)

        self.__file_menu = file_menu
        self.__plugins_menu = plugins_menu
        self.__windows_menu = windows_menu
        self.__projects_menu = projects_menu

        # File menu
        self.addMenu(self.__file_menu)
        self.addMenu(self.__plugins_menu)
        self.addMenu(self.__windows_menu)
        self.addMenu(self.__projects_menu)

        self.__file_menu.quit.connect(lambda: self.quit.emit())


MainMenuBarFactory = providers.Factory(
    MainMenuBar,
    file_menu=FileMenuFactory,
    plugins_menu=PluginsMenuFactory,
    windows_menu=WindowsMenuFactory,
    projects_menu=ProjectsMenuFactory,
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from PySide2.QtWidgets import QHeaderView, QTableView

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget


class PluginsTableView(QTableView):
    def __init__(self, parent: "QWidget" = None):
        super().__init__(parent)

        self.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
        self.horizontalHeader().setStretchLastSection(True)


PluginsTableViewFactory = providers.Factory(PluginsTableView)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

"""This package has several menus that can be used throughout the application on menu
bars and context menus."""

from .file_menu import FileMenu, FileMenuFactory
from .nodes_menu import NodesMenu, NodesMenuFactory
from .plugins_menu import PluginsMenu, PluginsMenuFactory
from .windows_menu import WindowsMenu, WindowsMenuFactory
from .projects_menu import ProjectsMenu, ProjectsMenuFactory

__all__ = [
    "FileMenu",
    "FileMenuFactory",
    "NodesMenu",
    "NodesMenuFactory",
    "PluginsMenu",
    "PluginsMenuFactory",
    "WindowsMenu",
    "WindowsMenuFactory",
    "ProjectsMenu",
    "ProjectsMenuFactory",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dial_gui',
 'dial_gui.event_filters',
 'dial_gui.main_window',
 'dial_gui.node_editor',
 'dial_gui.project',
 'dial_gui.utils',
 'dial_gui.widgets',
 'dial_gui.widgets.log',
 'dial_gui.widgets.menus',
 'dial_gui.widgets.plugin',
 'dial_gui.widgets.plugin.plugins_table']

package_data = \
{'': ['*']}

install_requires = \
['PySide2>=5.12.6,<6.0.0',
 'dependency-injector>=3.15.6,<4.0.0',
 'dial-core>=0.10a0,<0.11',
 'qimage2ndarray>=1.8.3,<2.0.0',
 'tensorflow==2.0.0b1']

setup_kwargs = {
    'name': 'dial-gui',
    'version': '0.7a0',
    'description': 'A node-based GUI for Deep Learning tasks',
    'long_description': "# dial-gui\nGUI for the __dial-core__ framework.\n\n| Build | Coverage | Quality | Version | Python | Docs | License |\n|-------|----------|---------|---------|--------|------|---------|\n| [![Build Status](https://travis-ci.com/dial-app/dial-gui.svg?branch=master)](https://travis-ci.com/dial-app/dial-gui) \t| [![codecov](https://codecov.io/gh/dial-app/dial-gui/branch/master/graph/badge.svg)](https://codecov.io/gh/dial-app/dial-gui) \t| [![Codacy Badge](https://api.codacy.com/project/badge/Grade/eb5224f0cc3f481aa5d419b4bfc86f41)](https://www.codacy.com/gh/dial-app/dial-gui?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=dial-app/dial-gui&amp;utm_campaign=Badge_Grade) \t| [![PyPI](https://img.shields.io/pypi/v/dial-gui)](https://pypi.org/project/dial-gui/) \t| [![PyPI - Python Version](https://img.shields.io/pypi/pyversions/dial-gui?color=green)](https://pypi.org/project/dial-gui/) \t| [![ReadTheDocs](https://readthedocs.org/projects/dial-gui/badge/?version=latest)](https://dial-gui.readthedocs.io/) \t| [![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0) |\n\n## Documentation\n\nThis project's documentation lives at [readthedocs.io](https://dial-gui.readthedocs.io).\n\n## License\n\nAll code is provided under the __GPL-3.0__ license. See [LICENSE](LICENSE) for more details.\n\n## Authors\n\n* **David Afonso (davafons)**: [Github](https://github.com/davafons) [Twitter](https://twitter.com/davafons)\n",
    'author': 'David Afonso',
    'author_email': 'davafons@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dial-app/dial-gui',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<3.8',
}


setup(**setup_kwargs)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

import dependency_injector.providers as providers

import logging

from PySide2.QtWidgets import QPlainTextEdit, QVBoxLayout, QWidget

from dial_core.utils import log


class LoggerTextboxWidget(logging.Handler, QWidget):
    """The LoggerTextboxWidget class provides a textbox widget that can also work as a
    logging handler for the Python logging system.

    Examples:
        textbox_logger = LoggerTextboxWidget()
        logging.getLogger().addHandler(textbox_logger)
    """

    def __init__(self, formatter: "logging.Formatter", parent: QWidget = None):
        logging.Handler.__init__(self)
        QWidget.__init__(self, parent)

        self.setFormatter(formatter)

        self._textbox = QPlainTextEdit(self)

        self.__setup_ui()

    @property
    def text(self) -> str:
        """Returns the text written on the textbox."""
        return self._textbox.toPlainText()

    def set_plain_text(self, text: str):
        """Replaces the textbox content with `text`."""
        self._textbox.setPlainText(text)

    def emit(self, record):
        """This method is called each time a logger emits a message. It applies the
        corresponding format and sends it to the textbox."""
        msg = self.format(record)
        self._textbox.appendPlainText(msg)

    def __setup_ui(self):
        """Configures the widget and layout settings."""
        # Log messages window must be readonly
        self._textbox.setReadOnly(True)

        # Add log textbox to layout without margins
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        layout.addWidget(self._textbox)

        self.setLayout(layout)


LoggerTextboxFactory = providers.Factory(LoggerTextboxWidget, formatter=log.FORMATTER)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

import dependency_injector.providers as providers
from dial_core.project import ProjectManager
from dial_core.utils import log
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QFileDialog, QMessageBox, QWidget

from .project_gui import ProjectGUI, ProjectGUIFactory

LOGGER = log.get_logger(__name__)


class ProjectManagerGUI(QWidget, ProjectManager):
    new_project_created = Signal(ProjectGUI)
    project_added = Signal(ProjectGUI)
    active_project_changed = Signal(ProjectGUI)
    project_removed = Signal(ProjectGUI)

    def __init__(self, default_project: "ProjectGUI", parent=None):
        QWidget.__init__(self, parent)
        ProjectManager.__init__(self, default_project)

    def open_project(self):
        LOGGER.debug("Opening dialog for pickling a file...")

        file_path = QFileDialog.getOpenFileName(
            QWidget(), "Open Dial project", "", "Dial Files (*.dial)"
        )[0]
        LOGGER.info("File path selected for opening: %s", file_path)

        if file_path:
            super().open_project(file_path)
        else:
            LOGGER.info("Invalid file path. Loading cancelled.")

    def save_project(self, project: "ProjectGUI") -> "ProjectGUI":
        try:
            return super().save_project(project)

        except ValueError:
            LOGGER.warning("Project doesn't have a file path set!")
            return self.save_project_as(project)

    def save_project_as(self, project: "ProjectGUI"):
        LOGGER.debug("Opening dialog for picking a save file...")

        selected_file_path = QFileDialog.getSaveFileName(
            QWidget(), "Save Dial project", "", "Dial Files (*.dial)"
        )[0]

        if not selected_file_path.endswith(".dial"):
            selected_file_path += ".dial"

        LOGGER.info("File path selected for saving: %s", selected_file_path)

        if selected_file_path:
            super().save_project_as(project, selected_file_path)
        else:
            LOGGER.info("Invalid file path. Saving cancelled.")

    def close_project(self, project: "ProjectGUI"):
        return_code = QMessageBox.warning(
            self,
            "The document has been modified",
            "Do you want to save your changes?",
            QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel,
        )

        if return_code == QMessageBox.Cancel:
            return

        if return_code == QMessageBox.Save:
            self.save_project(project)

        # The project is closed if we discard the project or we save it
        super().close_project(project)

    def closeEvent(self, event):
        for project in list(self.projects):
            self.close_project(project)

    def _new_project_impl(self) -> "ProjectGUI":
        new_project = super()._new_project_impl()
        self.new_project_created.emit(new_project)
        return new_project

    def _add_project_impl(self, project: "ProjectGUI") -> "ProjectGUI":
        super()._add_project_impl(project)

        self.project_added.emit(project)
        return project

    def _set_active_project_impl(self, index: int) -> "ProjectGUI":
        active_project = super()._set_active_project_impl(index)
        self.active_project_changed.emit(active_project)
        return active_project

    def _remove_project_impl(self, project: "ProjectGUI") -> "ProjectGUI":
        super()._remove_project_impl(project)
        self.project_removed.emit(project)
        return project


ProjectManagerGUISingleton = providers.Singleton(
    ProjectManagerGUI, default_project=ProjectGUIFactory
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from dial_core.utils import log
from dial_gui.node_editor import NodeEditorWindow
from dial_gui.project import ProjectManagerGUISingleton
from dial_gui.utils import application
from dial_gui.widgets.log import LoggerDialogFactory
from PySide2.QtCore import QSize
from PySide2.QtWidgets import QMainWindow, QTabBar, QTabWidget

from .main_menubar import MainMenuBarFactory

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget


LOGGER = log.get_logger(__name__)


class MainWindow(QMainWindow):
    """The main window for the program."""

    def __init__(
        self, main_menubar, project_manager, parent: "QWidget" = None,
    ):
        super().__init__(parent)

        # Initialize widgets
        self.__project_manager = project_manager
        self.__project_manager.setParent(self)

        self.__main_menu_bar = main_menubar
        self.__main_menu_bar.setParent(self)

        self.__tabs_widget = QTabWidget(self)
        self.__node_editor = NodeEditorWindow(
            tabs_widget=self.__tabs_widget,
            project_manager=self.__project_manager,
            parent=self,
        )

        # Configure ui
        self.__setup_ui()

        self.__main_menu_bar.quit.connect(self.close)

    def __setup_ui(self):
        # Set window title
        self.setWindowTitle(f"Dial {application.version()}")

        # Configure menu and status bars
        self.setMenuBar(self.__main_menu_bar)
        self.setStatusBar(self.statusBar())

        self.setCentralWidget(self.__tabs_widget)

        self.__tabs_widget.setMovable(True)
        self.__tabs_widget.setTabsClosable(True)

        self.__tabs_widget.tabCloseRequested.connect(
            lambda index: self.__tabs_widget.removeTab(index)
        )

        self.__tabs_widget.addTab(self.__node_editor, "Editor")

        # Remove "delete" button from the tab
        self.__tabs_widget.tabBar().tabButton(0, QTabBar.RightSide).deleteLater()
        self.__tabs_widget.tabBar().setTabButton(0, QTabBar.RightSide, None)

    def closeEvent(self, event):
        print("Main Menu close")
        self.__project_manager.closeEvent(event)

        super().closeEvent(event)

    def sizeHint(self) -> "QSize":
        """Returns the size of the main window."""
        return QSize(1000, 800)


MainWindowFactory = providers.Factory(
    MainWindow,
    main_menubar=MainMenuBarFactory,
    project_manager=ProjectManagerGUISingleton,
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

import dependency_injector.providers as providers
from PySide2.QtCore import Qt
from PySide2.QtGui import QColor, QPen

if TYPE_CHECKING:
    from .graphics_connection import GraphicsConnection
    from PySide2.QtGui import QPainter
    from PySide2.QtWidgets import QWidget, QStyleOptionGraphicsItem


class GraphicsConnectionPainter:
    def __init__(self, graphics_connection: "GraphicsConnection"):

        self.__graphics_connection = graphics_connection

        # Colors/Pens/Brushes
        self._color = QColor("black")

        self._default_pen = QPen(self._color)
        self._default_pen.setWidth(self.__graphics_connection.width)

    @property
    def color(self) -> "QColor":
        """Returns the color of this connection."""
        return self._color

    @color.setter
    def color(self, color: "QColor"):
        """Sets a new color for the connection, updating the QPen used for painting."""
        self._color = color
        self._default_pen.setColor(self._color)

    def highlight_color(self, toggle: bool):
        """Paints the connection with a highlighted color if toggled."""
        self._default_pen.setColor(self._color.lighter(150) if toggle else self._color)

    def paint(
        self,
        painter: "QPainter",
        option: "QStyleOptionGraphicsItem",
        widget: "QWidget" = None,
    ):
        """Paints the graphics connection item."""
        painter.setPen(self._default_pen)
        painter.setBrush(Qt.NoBrush)
        painter.drawPath(self.__graphics_connection.path())


GraphicsConnectionPainterFactory = providers.Factory(GraphicsConnectionPainter)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

import dependency_injector.providers as providers

from typing import TYPE_CHECKING

from dial_core.project import Project
from dial_gui.project import ProjectManagerGUI, ProjectManagerGUISingleton

from PySide2.QtWidgets import QMenu, QActionGroup, QAction
from dial_core.utils import log

if TYPE_CHECKING:
    from PySide2.QtWidgets import QWidget

LOGGER = log.get_logger(__name__)


class ProjectsMenu(QMenu):
    def __init__(
        self, project_manager_gui: "ProjectManagerGUI", parent: "QWidget" = None
    ):
        super().__init__("Projects", parent)

        self.__project_manager_gui = project_manager_gui
        self.__project_manager_gui.project_added.connect(self.__add_project_to_menu)
        self.__project_manager_gui.active_project_changed.connect(
            self.__check_active_project
        )
        self.__project_manager_gui.project_removed.connect(
            lambda: self.__generate_menu_from_projects()
        )

        self.__projects_actions_group = QActionGroup(self)

        self.__generate_menu_from_projects()
        self.__check_active_project(self.__project_manager_gui.active)

    def __generate_menu_from_projects(self):
        self.clear()

        for project in self.__project_manager_gui.projects:
            self.__add_project_to_menu(project)

    def __add_project_to_menu(self, project: "Project"):
        project_action = QAction(project.name, self)
        project_action.setCheckable(True)

        index = self.__project_manager_gui.projects.index(project)

        project_action.triggered.connect(
            lambda _=False, index=index: self.__project_manager_gui.set_active_project(
                index
            )
        )

        self.__projects_actions_group.addAction(project_action)
        self.addAction(project_action)

        LOGGER.debug("Created ProjectMenu Action (Index %s): %s", index, project_action)

    def __check_active_project(self, project: "Project"):
        index = self.__project_manager_gui.projects.index(project)

        self.actions()[index].setChecked(True)

        LOGGER.debug("Project with index %s is now active on menu: %s", index, project)


ProjectsMenuFactory = providers.Factory(
    ProjectsMenu, project_manager_gui=ProjectManagerGUISingleton
)



# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:

from typing import TYPE_CHECKING

from PySide2.QtCore import QEvent, QObject

if TYPE_CHECKING:
    from PySide2.QtWidgets import QGraphicsView
    from PySide2.QtGui import QWheelEvent


class ZoomEventFilter(QObject):
    """The ZoomEventFilter class provides an easy implementation of a zoom movement for
    a QGraphicsView object.

    This filter only works with a QGraphicsView object.

    Attributes:
        scale_factor_increment: Percentage incremented on each zoom step.

    Examples:
        view = QGraphicsView()
        zoom_event_filter = ZoomEventFilter()

        view.installEventFilter(zoom_event_filter)

        # Now the `view` can be zoomed using the mouse.
    """

    def __init__(self, parent: "QObject" = None):
        super().__init__(parent)

        self.scale_factor_increment = 0.2

    def eventFilter(self, obj: "QObject", event: "QEvent") -> bool:
        """Intercepts events emitted by `obj`. In this case, implements a zoom effect
        when the mouse wheel is moved up/down.

        This filter only works with a QGraphicsView object.

        Args:
            obj: The object (QGraphicsView) to pan.
            event: The generated event.

        Returns:
            if the event should be filtered out or not (Stopping it being handled by
            other objects).
        """
        if event.type() == QEvent.Wheel:
            return self.__zoom_view(obj, event)

        return super().eventFilter(obj, event)

    def __zoom_view(self, view: "QGraphicsView", event: "QWheelEvent") -> bool:
        """Zooms in/out the view using the mouse wheel.
        """
        if event.delta() > 0:
            view.scale(
                1.0 + self.scale_factor_increment, 1.0 + self.scale_factor_increment
            )
        else:
            view.scale(
                1.0 - self.scale_factor_increment, 1.0 - self.scale_factor_increment
            )

        return True
